import java.awt.*;
import java.applet.*;

class CollideBall
{
	int width, height;
	public static final int diameter=30;
	//coordinates and value of increment
	double x, y, xinc, yinc, coll_x, coll_y;
	boolean collide;
	Color color;
	Graphics g;
	
	//the constructor
	public CollideBall(int w, int h, int x, int y, double xinc, double yinc, Color c)
	{
		width=w;
		height=h;
		this.x=x;
		this.y=y;
		this.xinc=xinc;
		this.yinc=yinc;		
		color=c;		
	}
	
	public double getCenterX() {return x+diameter/2;}
	public double getCenterY() {return y+diameter/2;}
	
	public void move()
	{
		if (collide)
		{			
			double xvect=coll_x-getCenterX();
			double yvect=coll_y-getCenterY();

			if((xinc>0 && xvect>0) || (xinc<0 && xvect<0))
				xinc=-xinc;

			if((yinc>0 && yvect>0) || (yinc<0 && yvect<0))
				yinc=-yinc;
	
			collide=false;
		}
	
		x+=xinc;
    	y+=yinc;

		//when the ball bumps against a boundary, it bounces off
    	if(x<0 || x>width-diameter)
    	{
    		xinc=-xinc;
	  		x+=xinc;
    	}
	    
	   if(y<0 || y>height-diameter)
    	{
    		yinc=-yinc;
	  		y+=yinc;
    	}	
	}
	
	public void hit(CollideBall b)
	{
		if(!collide)
		{
			coll_x=b.getCenterX();
			coll_y=b.getCenterY();
			collide=true;
		}
	}
	
	public void paint(Graphics gr)
	{	
		g=gr;
		g.setColor(color);
		//the coordinates in fillOval have to be int, so we cast
		//explicitly from double to int
		g.fillOval((int)x,(int)y,diameter,diameter);					
	}	
}

public class Project33 extends Applet implements Runnable 
{
	Thread runner;	
	Image Buffer;
   Graphics gBuffer;		
   CollideBall ball[];
   AudioClip clickSound;
   boolean sound;
   //how many balls?
   static final int MAX=5;
	
	public void init() 
	{			
		Buffer=createImage(size().width,size().height);
      gBuffer=Buffer.getGraphics();					
      
      ball=new CollideBall[MAX];
            
      int w=size().width;
      int h=size().height;
      
      try
	   {
			clickSound=getAudioClip(getCodeBase(),"click.au");			
		}
		catch (Exception e){}    
      
      //our balls have different start coordinates, increment values
      //(speed, direction) and colors
      ball[0]=new CollideBall(w,h,50,20,1.5,2.0,Color.orange);
      ball[1]=new CollideBall(w,h,60,100,2.0,-3.0,Color.red);
		ball[2]=new CollideBall(w,h,15,70,-2.0,-2.5,Color.pink);
		ball[3]=new CollideBall(w,h,150,60,-2.7,-2.0,Color.cyan);
		ball[4]=new CollideBall(w,h,210,30,2.2,-3.5,Color.magenta);
	}	

	public void start()
	{
		if (runner == null) 
		{
			runner = new Thread (this);
         runner.start();
      }
   }

   public void stop()
	{
      if (runner != null) 
		{
         	runner.stop();
           	runner = null;
     	}
   }

	public void run()
	{
		while(true) 
		{
			//Thread sleeps for 15 milliseconds here					
			try {runner.sleep(15);}
            	catch (Exception e) { }			
						
			//move our balls around
			for(int i=0;i<MAX;i++)
				ball[i].move();
				
			handleCollision();
			
         repaint();	                        
		}	
	}
	
	boolean collide(CollideBall b1, CollideBall b2)
	{
		double wx=b1.getCenterX()-b2.getCenterX();
		double wy=b1.getCenterY()-b2.getCenterY();

		//we calculate the distance between the centers two
		//colliding balls (theorem of Pythagoras)
		double distance=Math.sqrt(wx*wx+wy*wy);

		if(distance<b1.diameter)
		{
			if(sound)clickSound.play();
			return true;
		}
		
			return false;	
	}
	
	public boolean mouseDown(Event evt,int x,int y)
	{
		sound^=true;

		return true;
	}
	
	private void handleCollision()
	{	
		//we iterate through all the balls, checking for collision
		for(int i=0;i<MAX;i++)
			for(int j=0;j<MAX;j++)
				{
					if(i!=j)
					{										
						if(collide(ball[i], ball[j]))
						{			
							ball[i].hit(ball[j]);
							ball[j].hit(ball[i]);
						}
					}
				}	
	}
	
	public void update(Graphics g)
	{
		paint(g);
  	}
        
  	public void paint(Graphics g) 
	{
		//draw a gray background and a grid of lines
		gBuffer.setColor(Color.gray);
		gBuffer.fillRect(0,0,size().width,size().height);
		
		gBuffer.setColor(Color.lightGray);		
		
		for(int x=0;x<size().width;x+=50)
			gBuffer.drawLine(x,0,x,size().height);
			
		for(int y=0;y<size().height;y+=50)
			gBuffer.drawLine(0,y,size().width,y);
			
		gBuffer.setColor(Color.orange);
		
		if(sound)
			gBuffer.drawString("Click to turn sound OFF", 10, 20);
		else
			gBuffer.drawString("Click to turn sound ON", 10, 20);				

		//paint our balls
		for(int i=0;i<MAX;i++)
				ball[i].paint(gBuffer);
		
		g.drawImage (Buffer,0,0, this);				
  	}	
}

